/*

    GDS7 v0.1 interactive digital image transfer software for DS-7 camera
    Copyright (C) 1998 Matthew G. Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#include "ds7lib.h"
#include "gds7.h"
#include "ops.h"
#include "ui.h"
#include "callbacks.h"

char* camdev;

/**********************************************************************
  Misc Functions
 **********************************************************************/

gint is_selected(int i){
  return(GTK_TOGGLE_BUTTON(pics[i-1].frame)->active);
};

/***********************************************************************

  Block Operations: multiple pictures

***********************************************************************/

/* Get all selected thumbnails */
void getthumbs(GtkWidget *widget, gpointer *data){
  int i,getall;

  getall=0;
  for (i=1;i<=numpix;i++){   
    getall|=is_selected(i);
  };
  getall=!getall;
  /* 
     If getall is true, none are selected 
     and ALL thumbs will be downloaded 
  */
  clear_status();

  for (i=1;(i<=numpix)&&get_status();i++){   
    if ((is_selected(i)||getall) &&!(pics[i-1].status&STAT_THUMB)) 
      thumbtrans(pics[i-1].picnum);
  };
};



/* 
   Save all selected pictures, or all if none are selected
*/
void allsave(GtkWidget *widget, gpointer *data){
  int i,getall;

  getall=0;
  for (i=0;i<numpix;i++){   
    getall|=(is_selected(i+1));
  };
  getall=!getall;
  /* 
     If getall is true, none are selected 
     and ALL thumbs will be downloaded 
  */
  clear_status();
  for (i=0;(i<numpix)&&get_status();i++){   
    if ((is_selected(i+1) || getall)  &&!(pics[i].status&STAT_SAVED)) {
      getpic(pics+i);
    };
  };
};

/* Deselect all thumbnails */
void deselect (GtkWidget *widget, gpointer *data){
  int i;
  for (i=0;i<numpix;i++) {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pics[i].frame),FALSE);
    color_selected(pics+i);
  };
};


/***********************************************************************

Single Picture Operations

 ***********************************************************************/


/* 
   Download the picture to a temp file and call the previewer 
*/
void preview(GtkWidget *widget, gpointer data){
  char *temp,*realname,syscmd[200];
  /* Snag poppic before the user moves the cursor or something */
  struct picent *pic=poppic;

  if (pic==NULL) return;
  temp=tempnam(NULL,"gds7p");
  realname=strdup(gtk_entry_get_text(GTK_ENTRY(pic->namebox)));
#ifdef DEBUG
  printf("Realname returned %s\n",realname);
#endif
  gtk_entry_set_text(GTK_ENTRY(pic->namebox),temp);
  getpic(pic);
  gtk_entry_set_text(GTK_ENTRY(pic->namebox),realname);
  free(realname);

  /* Inelegant way to delete the file upon previewer quit,WFM */
  sprintf(syscmd,"(%s %s ; rm %s)&",preview_cmd,temp,temp);
  system(syscmd);
  printf("We're back\n");
};


/*
  Get a thumbnail from the camera and place it in the
  imagelist
*/
void thumbtrans(int i){
  GdkImlibImage *im,*ims;
  GdkImlibColorModifier mod;
  GdkPixmap *p,*m;
  char *temp;
  m=NULL;
  
  temp=tempnam(NULL,"thumb");
  if (!get_thumb(i,temp,progcb)) return;

      im=gdk_imlib_load_image(temp);
      gdk_imlib_get_image_modifier(im,&mod);
      mod.gamma*=thumbgam;
      gdk_imlib_set_image_modifier(im,&mod);
      gdk_imlib_apply_modifiers_to_rgb(im);

      ims=gdk_imlib_clone_scaled_image(im,im->rgb_width*thumbscale,
				   im->rgb_height*thumbscale);
      gdk_imlib_render(ims,ims->rgb_width,ims->rgb_height);
      p=gdk_imlib_move_image(ims);

      gtk_pixmap_set(GTK_PIXMAP(pics[i-1].preview),p,m);

      /* Assure Timely Display of the image */
      gtk_sync();

      /* Clean Up */
      gdk_imlib_free_pixmap(p);
      gdk_imlib_kill_image(im);
      gdk_imlib_kill_image(ims);

      pics[i-1].status|=STAT_THUMB;
  remove(temp);
  free(temp);
};

/***********************************************************************

  Camera Operations

 ***********************************************************************/

/*
  Open the camera and upgrade the link speed
 */
int cam_init(GtkWidget *widget, gpointer *data){
  char tempstr[30];
  int i;

  /* Close link if already opened */
  if(init) {
    close_ds7();
    free(pics);
    pics=NULL;
  };

  if ((numpix=open_ds7(camdev))==0){
    popup_dialog("Could not open camera!\n Check power, settings, cables");
    return 1;
  };

  sprintf(tempstr,"%d",numpix);
  gtk_statusbar_pop(GTK_STATUSBAR(pixcount),1);
  gtk_statusbar_pop(GTK_STATUSBAR(camid),1);
  gtk_statusbar_push(GTK_STATUSBAR(pixcount),1,tempstr);
  gtk_statusbar_push(GTK_STATUSBAR(camid),1,thecam_info->cam_id);

  if(set_speed(baudrate)) {
    printf("Error Changing Speed\n");
    exit(1);
  };

  init=1;

  make_frames();

  return 0;
};

